import logging


class LoggingMixin:
    def execute(self, *args, **options):
        logging.root.setLevel(
            {
                0: logging.ERROR,
                1: logging.WARNING,
                2: logging.INFO,
                3: logging.DEBUG,
            }.get(options["verbosity"])
        )

        ch = logging.StreamHandler()
        formatter = logging.Formatter(logging.BASIC_FORMAT)
        ch.setFormatter(formatter)

        logging.root.addHandler(ch)
        return super().execute(*args, **options)



import uuid
import typing
from functools import lru_cache, wraps

from django.apps import apps
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from paho.mqtt import publish
from .decorators import json_payload


if typing.TYPE_CHECKING:
    from paho.mqtt.client import PayloadType


__all__ = [
    "default_client_id",
    "multiple",
    "single",
]


@lru_cache(maxsize=1)
def default_client_id():
    if apps.is_installed("django.contrib.sites"):
        return (get_current_site(None).domain + "-%d" % uuid.uuid4().int)[:32]
    return ""


@wraps(publish.multiple)
def multiple(msgs: "publish.MessagesList", client_id=None, **kwargs):
    if settings.MQTT_USER and settings.MQTT_PASS:
        kwargs["auth"] = {
            "username": settings.MQTT_USER,
            "password": settings.MQTT_PASS,
        }
    if client_id is None:
        client_id = default_client_id()
    return publish.multiple(
        msgs,
        hostname=settings.MQTT_HOST,
        port=settings.MQTT_PORT,
        client_id=client_id,
        **kwargs,
    )


@json_payload
@wraps(publish.single)
def single(
    topic: str,
    payload: "PayloadType" = None,
    qos: int = 0,
    retain: bool = False,
    **kwargs,
) -> None:
    """
    Wrapped version of single, supporting AUTH and json payload
    """
    msg: "publish.MessageDict" = {
        "topic": topic,
        "payload": payload,
        "qos": qos,
        "retain": retain,
    }
    return multiple([msg], **kwargs)



import json
import logging

from django.contrib.auth import authenticate
from django.contrib.auth.signals import user_login_failed
from django.http import HttpResponseForbidden, JsonResponse
from django.urls import path
from django.views import View
from django.views.decorators.csrf import csrf_exempt

logger = logging.getLogger(__name__)


# https://docs.vernemq.com/plugindevelopment/webhookplugins#auth_on_register
class OnRegister(View):
    def post(self, request):
        data = json.loads(request.body.decode("utf-8"))
        if authenticate(request, username=data["username"], password=data["password"]):
            return JsonResponse({"result": "ok"})
        logger.debug("Invalid login for %s", data["username"])
        user_login_failed.send(
            sender=__name__,
            credentials={key: data[key] for key in data if key not in ["password"]},
            request=request,
        )
        return HttpResponseForbidden()


# https://docs.vernemq.com/plugindevelopment/webhookplugins#auth_on_subscribe
class OnSubscribe(View):
    def post(self, request):
        # TODO Proper auth
        return JsonResponse({"result": "ok"})


# https://docs.vernemq.com/plugindevelopment/webhookplugins#auth_on_publish
class OnPublish(View):
    def post(self, request):
        # TODO Proper auth
        return JsonResponse({"result": "ok"})


urlpatterns = [
    path("auth_on_register", csrf_exempt(OnRegister.as_view())),
    path("auth_on_subscribe", csrf_exempt(OnSubscribe.as_view())),
    path("auth_on_publish", csrf_exempt(OnPublish.as_view())),
]



import sys

from paho.mqtt.publish import single

from django.conf import settings
from django.core.management.base import BaseCommand, CommandParser

from dmqtt.shortcuts import default_client_id


class Command(BaseCommand):
    def add_arguments(self, parser: CommandParser):
        mqtt = parser.add_argument_group("mqtt server arguments")
        mqtt.add_argument("topic", help="MQTT Topic to send to")
        mqtt.add_argument("-u", "--user", default=settings.MQTT_USER)
        mqtt.add_argument("-P", "--password", default=settings.MQTT_PASS)
        mqtt.add_argument("-H", "--host", default=settings.MQTT_HOST)
        mqtt.add_argument("--client-id", default=default_client_id())
        mqtt.add_argument("--port", default=settings.MQTT_PORT, type=int)
        mqtt.add_argument("--qos", default=0, type=int)
        mqtt.add_argument("--retain", action="store_true")

    def handle(self, topic, **kwargs):
        payload = sys.stdin.read().strip().encode("utf8")

        single(
            topic=topic,
            payload=payload,
            qos=kwargs["qos"],
            retain=kwargs["retain"],
            client_id=kwargs["client_id"],
            hostname=kwargs["host"],
            port=kwargs["port"],
            auth={"username": kwargs["user"], "password": kwargs["password"]},
        )



from django.core.checks import Error, register
from django.conf import settings


setting_names = [
    "MQTT_HOST",
    "MQTT_USER",
    "MQTT_PASS",
    "MQTT_PORT",
]


@register()
def settings_check(app_configs, **kwargs):
    for name in setting_names:
        if not hasattr(settings, name):
            yield Error(
                msg=f"Missing setting {name}",
                hint=f"Need to add {name} to {settings.SETTINGS_MODULE}",
                obj=settings,
            )



from django.utils.module_loading import import_string
from django.conf import settings
import logging

import paho.mqtt.client as mqtt


from dmqtt.decorators import json_payload
from dmqtt.signals import connect, message


class Client(mqtt.Client):
    def on_connect(self, client, userdata, flags, rc):
        connect.send_robust(client, userdata=userdata, flags=flags, rc=rc)

    def on_message(self, client, userdata, msg):
        if logging.root.isEnabledFor(logging.INFO):
            try:
                if msg.retain:
                    print("*", end=" ")
                print(msg.topic, msg.payload.decode("utf8"))
            except UnicodeDecodeError:
                print(msg.topic, "** Unknown Encoding **")
        message.send_robust(client, userdata=userdata, msg=msg)

    publish = json_payload(mqtt.Client.publish)


def get_client() -> type[Client]:
    class_path = getattr(settings, "MQTT_CLIENT", "dmqtt.client.Client")
    return import_string(class_path)



import functools
import json
import logging
import re
import django.dispatch

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from paho.mqtt.client import MQTTMessage


logger = logging.getLogger(__name__)

# userdata, flags, rc
connect = django.dispatch.Signal()
# userdata, msg
message = django.dispatch.Signal()


def convert_wildcards(mqtt_topic) -> re.Pattern[str]:
    """Convert MQTT wildcards to regex pattern.
    '#' becomes '.*' (multi-level)
    '+' becomes '[^/]+' (single-level)
    """
    pattern = re.escape(mqtt_topic)
    pattern = pattern.replace("\\#", ".*")
    pattern = pattern.replace("\\+", "[^/]+")
    return re.compile("^" + pattern + "$")


def topic(matcher, as_json=True, **extras):
    regex_pattern = convert_wildcards(matcher)

    def wrap(func):
        @functools.wraps(func)
        def inner(msg: "MQTTMessage", **kwargs):
            if regex_pattern.match(msg.topic):
                logger.debug("Matched %s for %s", regex_pattern, func)
                if as_json:
                    kwargs["data"] = json.loads(msg.payload.decode("utf8"))
                func(topic=msg.topic, msg=msg, **kwargs)

        message.connect(inner, **extras)
        return inner

    return wrap


def regex(pattern, *, as_json=True, **extras):
    matcher = re.compile(pattern)

    def wrap(func):
        @functools.wraps(func)
        def inner(msg: "MQTTMessage", **kwargs):
            match = matcher.match(msg.topic)
            if match:
                logger.debug("Matched %s for %s", match, func)
                if as_json:
                    kwargs["data"] = json.loads(msg.payload.decode("utf8"))
                func(topic=msg.topic, match=match, msg=msg, **kwargs)

        message.connect(inner, **extras)
        return inner

    return wrap



import json
import logging

from django.contrib.auth import authenticate
from django.contrib.auth.signals import user_logged_in
from django.http import HttpResponseForbidden, JsonResponse
from django.urls import path
from django.views import View
from django.views.decorators.csrf import csrf_exempt

logger = logging.getLogger(__name__)


# https://github.com/iegomez/mosquitto-go-auth#http
class GetUser(View):
    def post(self, request):
        data = json.loads(request.body.decode("utf-8"))
        user = authenticate(
            request,
            username=data["username"],
            password=data["password"],
        )
        if user is None:
            return HttpResponseForbidden()

        user_logged_in.send(sender=user.__class__, request=request, user=user)

        return JsonResponse({"result": "ok"})


class ACLCheck(View):
    def post(self, request):
        # TODO Proper auth
        return JsonResponse({"result": "ok"})


urlpatterns = [
    path("getuser", csrf_exempt(GetUser.as_view())),
    path("aclcheck", csrf_exempt(ACLCheck.as_view())),
]



from . import LoggingMixin

from django.conf import settings
from django.core.management.base import BaseCommand
from django.test import override_settings

from dmqtt.shortcuts import default_client_id
from dmqtt.client import Client, get_client


class Command(LoggingMixin, BaseCommand):
    def get_mqtt_client(self) -> type[Client]:
        return get_client()

    def add_arguments(self, parser):
        mqtt = parser.add_argument_group("mqtt server arguments")
        mqtt.add_argument("-u", "--user", default=settings.MQTT_USER)
        mqtt.add_argument("-P", "--password", default=settings.MQTT_PASS)
        mqtt.add_argument("-H", "--host", default=settings.MQTT_HOST)
        mqtt.add_argument("--port", default=settings.MQTT_PORT, type=int)
        mqtt.add_argument("--client-id", default=default_client_id())

        celery = parser.add_argument_group("celery arguments")
        celery.add_argument("--eager", action="store_true")

    def handle(self, eager, **kwargs):
        client = self.get_mqtt_client()(client_id=kwargs["client_id"])
        client.enable_logger()  # Use Python logging
        client.username_pw_set(kwargs["user"], password=kwargs["password"])
        # TODO Fix
        # client.tls_set()
        client.connect(kwargs["host"], kwargs["port"], 60)

        try:
            from setproctitle import setproctitle
        except ImportError:
            pass
        else:
            if client._client_id:
                setproctitle(f"[mqtt] {client._client_id.decode('utf8')}")

        with override_settings(CELERY_TASK_ALWAYS_EAGER=eager):
            # Blocking call that processes network traffic, dispatches callbacks and
            # handles reconnecting.
            # Other loop*() functions are available that give a threaded interface and a
            # manual interface.
            try:
                client.loop_forever()
            except KeyboardInterrupt:
                client.disconnect()



import json
from functools import wraps


from typing import TypeVar, ParamSpec, Callable

T = TypeVar("T")
P = ParamSpec("P")

# The JSONEncoder from DRF handles quite a few types, so we default to that
# if available and if not fallback to the Django one which still handles some
# extra types
try:
    from rest_framework.utils.encoders import JSONEncoder
except ImportError:
    from django.core.serializers.json import DjangoJSONEncoder as JSONEncoder


def json_payload(func: Callable[P, T]) -> Callable[P, T]:
    """
    Decorator to add json support to MQTT publish methods
    """

    @wraps(func)
    def __inner(*args, **kwargs):
        if "json" in kwargs:
            data = kwargs.pop("json")
            kwargs["payload"] = json.dumps(data, cls=JSONEncoder).encode("utf8")

        return func(*args, **kwargs)

    return __inner
